%-------------------------------------------------------------------------%
% Background
% Dispersion analyis under the Bloch boundary condition
% Created by Lei, TU/e, 2017
%-------------------------------------------------------------------------%

function [Normk, Freq, U] = blochAnalysis(K,M,boundaryNodes,W,nNodes,DofR,extendDofs,key_cons,DirWave,Nset,eModes)

% Initialize
nModes = Nset(1)+eModes;                                 % total number of selected modes
nSamples = Nset(2);                                % sampling number
freqTh = Nset(4);                                  % to filter low frequency

kmax = pi / W;
k = linspace(0,kmax,nSamples);                     % wave number
Normk = k / kmax;

kX = DirWave(1) * k;
kY = DirWave(2) * k;

nNodesX = length(boundaryNodes{1});                % number of X-boundary nodes
nNodesY = length(boundaryNodes{3});                % number of Y-boundary nodes

% Alloctae the nodes
PresNodes = [1 boundaryNodes{1} boundaryNodes{3}]'; % prescibed nodes
DepNodes = [[2 3 4] boundaryNodes{2} boundaryNodes{4}]';  % dependent nodes
ReNodes = setdiff([1:nNodes]',[PresNodes;DepNodes]);                % independent nodes

IndNodes = [PresNodes; ReNodes];
IndDofs = getDofs(IndNodes);
IndDofs = [IndDofs; DofR];
IndDof = length(IndDofs);

DepDofs = getDofs(DepNodes);
DepDof = length(DepDofs);

totDof = IndDof+DepDof;

% Get the frequency
U = cell(1,nSamples);
Freq = zeros(nModes,nSamples);

% Get the matrix Cm 
if key_cons == 0
   % no extra constriants (cruciform)
   consDofs = [];
else
   consDofs = extendDofs(1);
   % constraint X displacements of the resonantor
   
end

consDof = length(consDofs); 
Cm = zeros(IndDof,IndDof-consDof);
j = 1;
for i = 1:IndDof
    if any(IndDofs(i) == consDofs)
        continue
    end
    Cm(i,j) = 1;
    j = j+1;
    
end
IndFix = ismember(IndDofs,consDofs);
IndFllow = ismember(IndDofs,extendDofs([3 5]));


for i = 1:nSamples    
    % form the dependency matrix Cdi
    kx = kX(i);
    ky = kY(i);
    Cdi = funCdi(IndDofs,DepDofs,nNodesX,nNodesY,kx,ky,W);
    
    % partition K and M as [ii,id;di,dd]
    Kii = K(IndDofs,IndDofs);
    Kid = K(IndDofs,DepDofs);
    Kdi = K(DepDofs,IndDofs);
    Kdd = K(DepDofs,DepDofs);

    Mii = M(IndDofs,IndDofs);
    Mid = M(IndDofs,DepDofs);
    Mdi = M(DepDofs,IndDofs);
    Mdd = M(DepDofs,DepDofs);
    
    % remove dependent nodes (KEff*ui + MEff*ai = fiEff)
    KEff = Kii +Kid *Cdi + Cdi'*Kdi +Cdi'*(Kdd*Cdi);
    MEff = Mii +Mid *Cdi + Cdi'*Mdi +Cdi'*(Mdd*Cdi);

    KFF = Cm' * KEff * Cm;                           % contrain only free DoFs
    MFF = Cm' * MEff * Cm;

    % eigenvalue analysis
    [freeU, Omega2] = eigs(KFF,MFF,nModes,'sm');      % results at free DOFs

    IndU = Cm * freeU;
    if key_cons == 1
       IndU(IndFix,:) = sum(IndU(IndFllow,:))/2;  % depends on detailed requirement
    end
    DepU = Cdi * IndU;
    
    % collect frequency
    [Freq(:,i), idFreq]= sort(abs(sqrt(diag(Omega2)))/2/pi);
    
    % collect results at all nodes
    totU = zeros(totDof,nModes);
    totU(IndDofs,:) = IndU(:,idFreq);
    totU(DepDofs,:) = DepU(:,idFreq);
    
    U{i} = totU; 
    
    pcomp=round(i/nSamples*100);
    fprintf('Bloch Analysis: %d%% completed \n ',pcomp)
            
end

% filter too low frequencies
freq =  Freq(:,end);
[~,index] = find(freq'>=freqTh);
Freq = Freq(index,:);

for i = 1:nSamples 
    U{i} = U{i}(:,index);
end

end


function Cdi = funCdi(IndDofs,DepDofs,nNodesX,nNodesY,kx,ky,L)

% Ind: 1 left bottom others, Dep: 2 3 4 right top
IndDof = length(IndDofs);
DepDof = length(DepDofs);

% Initialize
Cdx = exp(1i*kx*L)*[ 1 0; 0 1];  
Cdy = exp(1i*ky*L)*[ 1 0; 0 1];  
Cdxy = exp(1i*(kx+ky)*L) * [1 0; 0 1];
Cdi = zeros(DepDof,IndDof);

% For nodes 2 3 4
Cdi(1:6,1:2) = [Cdx;Cdxy;Cdy];

% For right nodes
for iright = 1:nNodesX
    ncurrent = 3+iright;
    Cdi(2*ncurrent-1:2*ncurrent,2*(ncurrent-2)-1:2*(ncurrent-2)) = Cdx;

end

% For the top nodes
for itop = 1:nNodesY
    ncurrent = 3+itop+nNodesX;
    Cdi(2*ncurrent-1:2*ncurrent,2*(ncurrent-2)-1:2*(ncurrent-2)) = Cdy;
    
end

end