%-------------------------------------------------------------------------%
% Background
% Main function for the cell analysis (with a single resonator)
% Created by Lei, TU/e, 2017

% Notes
% Model: Plane stress or Plane strain (linear elasticity)
% Boundary: periodic boundary conditions
% Element type: 3-node triangle or 4-node quadriangle
% Unit system (unit thickness): N,Pa,m,kg;
% Unit thickness
% sparse matrix solving method

% Key input
% material parameters, material model
% meshfile, result filenames

%-------------------------------------------------------------------------%

clear,clc
warning off all

MAT_dynamic_parameters =['Dynamic_parameters.mat'];
MAT_dispersion_comparison =['Dispersion_comparison.mat'];

%% Settings

% matrix
E = 25e9;                            % modulus
mu = 0.3;                            % poisson ratio
rho = 2e3;                           % density
W = 20e-3;                           % width

MM = rho*(W^2 - pi*(W*0.25)^2);            % mass

Kc = E/3/(1-2*mu);
Gc = E/2/(1+mu);

Model = 'Planestrain';                % material model
C = planeModel(Kc,Gc,Model);          % tangent

% resonator (case D is tested)
key_R = 'D';                          % select the resonator configuration

switch key_R                        % point to three important input parameters
    case 'A'
        paraCont = [1 0.5 0];
    case 'B'
        paraCont = [2 0.7 2];
    case 'C'
        paraCont = [1 0.5 0];
    case 'D'
        paraCont = [3 0.7 2];
    
end

nr = paraCont(1);                               % number of resonators
key_cons = 0;                         % 1: add extra constriants on the nodal dispalcements
Ramp = W * 0;                         % shown displacement amplitude
fImg = 0.25;                          % to scale the imaginary axis
coordsRef = [0.5 0.5] * W;            % reference postion, default at the center

r = 0.05 * W;                         % radius
rs = 1 * r;                          % rotation radius of the hinge joint, case C
alpha = 15/180*pi;                   % reference 21.8deg
shHin = tan(alpha)*0.25;
coordsHin = [0.5 0.5+shHin;                 % hinge joint positions, case B and D
            0.5 0.5-shHin] * W;
coordsRn = [0.5 paraCont(2);  
            0.5 0.3;                       
            0.5 0.50] * W;            % resonator positions, one for cases A and C, two for B, three for D

KR = 0.005 *E * [1 1e-8];             % spring stiffness per unit thickness at horizontal state (0.01-10, 0.01-20, 0.02-10)
KR = [KR 0.5*KR];                     % extra springs for type D
rhoR = 50 * [rho rho];                % density
MR = rhoR * (pi*r^2);                 % single mass
rhom = rho*(W^2-pi*(0.25*W)^2)/W/W;

% mesh file
meshfile = 'Matrix2D_Q2.txt';

% vibration analysis
DirWave = [1 0];
Freqlim = 0.5e4;
RANGE = '-1 to 1';                    % k range

nModes = 6;                           % mode number
eModes = paraCont(3);                           % mode number to be removed
nSamples = 100;                       % for Bloch analysis
Nomega = 500*nSamples;                % for homogenization model 
freqTh = 100;                         % frequency threshold

Nset = [nModes nSamples Nomega freqTh];


%% Read the mesh and constraints

[coords,elems,presNodes,boundaries] = readMeshMicro(meshfile);

coords = coords * W;

nNodes = length(coords);               % total node number
totDof = 2*nNodes;                     % total DOFs

nElems = length(elems);                % total element number
nElemNodes = length(elems(1,:));

presDofs = getDofs(presNodes);         % prescribed DOFs       
presDof = length(presDofs);


%% Approximate volume

% X-Y sizes
maxcoordsX = max(coords(:,1));
mincoordsX = min(coords(:,1));
maxcoordsY = max(coords(:,2));
mincoordsY = min(coords(:,2));
rangeX = maxcoordsX - mincoordsX;
rangeY = maxcoordsY - mincoordsY;

V = rangeX*rangeY;


%% Select integration scheme

coordsElem = coords(elems(1,:),:);     % element sample
order = 0;                             % default value (1,2 are optional)

% element properties
[nElemNodes,nIntElem,dVElem] = getElemProp(coordsElem,order);
totInt = nIntElem*nElems;              % total integration points

%% Collect tangent stiffness

CInt = cell(1,totInt);
for iInt =1:totInt
    CInt{iInt} = C;
end


%% Assemble system stiffness and mass matrice

% matrix stiffness and mass K*u + M*a = fext,a: acceleration
[K,M] = assembleKeMe(coords,elems,CInt,rho,nIntElem,dVElem,order);

e = 0.05;
% coupNodes = [24 20 18 22];   % for Q0 structure

switch key_R
case 'A'
% get coupling nodes (horizontal and vertical)
CoupX = min(coords(:,1))+ rangeX*0.5;
CoupY = min(coords(:,2))+ rangeY/2;

IndexX = find((1-e) * CoupY <= coords(:,2) & coords(:,2)<= (1+e) * CoupY );
IndexY = find((1-e) * CoupX <= coords(:,1) & coords(:,1)<= (1+e) * CoupX );
coupNodes = [IndexX(2) IndexX(1) IndexY(1) IndexY(2)]; % left right bottom top

% add a cruciform resonator
[K,M,rDofs,extendElems,extendDofs] = addResonatorA(K,M,KR,MR,coords,coupNodes,coordsRn(1,:));

case 'B'
%  get coupling nodes (horizontal)
CoupY = min(coords(:,2))+ rangeY*0.5;
IndexX = find((1-e) * CoupY <= coords(:,2) & coords(:,2)<= (1+e) * CoupY );
coupNodes = [IndexX(2) IndexX(1)];   % left right

% add a herringboned resonator
[K,M,rDofs,extendElems,extendDofs,Alpha] = addResonatorB(K,M,KR,MR,nr,coords,coupNodes,coordsHin);

case 'C'
CoupX = min(coords(:,1))+ rangeX*0.5;
CoupY = min(coords(:,2))+ rangeY/2;

IndexX = find((1-e) * CoupY <= coords(:,2) & coords(:,2)<= (1+e) * CoupY );
IndexY = find((1-e) * CoupX <= coords(:,1) & coords(:,1)<= (1+e) * CoupX );
coupNodes = [IndexX(2) IndexX(1) IndexY(1) IndexY(2)]; % left right bottom top

[K,M,rDofs,extendElems,extendDofs,Alpha,Theta,coordsHin] = addResonatorC(K,M,KR,MR,coords,coupNodes,coordsRn(1,:),rs,r);

case 'D'
%  get coupling nodes (horizontal)
CoupY = min(coords(:,2))+ rangeY*0.5;
IndexX = find((1-e) * CoupY <= coords(:,2) & coords(:,2)<= (1+e) * CoupY );
coupNodes = [IndexX(2) IndexX(1)];    % left right
% coupNodes = [IndexX(4) IndexX(3)];    % left right

[K,M,rDofs,extendElems,extendDofs,Alpha] = addResonatorD(K,M,KR,MR,nr,coords,coupNodes,coordsHin);  
    
end


%% Independent, dependent and free nodes

[IndNodes,DepNodes,freeNodes,Cdi] = relationInd(nNodes, presNodes, boundaries);

IndDofs = getDofs(IndNodes);           % basic independent DOFs
IndDofs = [IndDofs;rDofs];              % extended DOFs
IndDof = length(IndDofs);              % number

DepDofs = getDofs(DepNodes); 
DepDof = length(DepDofs);

freeDofs =  getDofs( freeNodes );
freeDofs = [freeDofs;rDofs];
freeDof = length(freeDofs);

Cdi = [Cdi zeros(DepDof,length(rDofs))];           % extended Cdi


%% partition K and M as [ii,id;di,dd] 
Kii = K(IndDofs,IndDofs);
Kid = K(IndDofs,DepDofs);
Kdi = K(DepDofs,IndDofs);
Kdd = K(DepDofs,DepDofs);

Mii = M(IndDofs,IndDofs);
Mid = M(IndDofs,DepDofs);
Mdi = M(DepDofs,IndDofs);
Mdd = M(DepDofs,DepDofs);

% remove dependent nodes (KEff*ui + MEff*ai = fiEff)
KEff = Kii +Kid *Cdi + Cdi'*Kdi +Cdi'*(Kdd*Cdi);
MEff = Mii +Mid *Cdi + Cdi'*Mdi +Cdi'*(Mdd*Cdi);


%% Partition KEff and MEff as[pp,pf;fp,ff]

% partition KEff
Kpp = KEff(1:presDof,1:presDof);
Kpf = KEff(1:presDof,presDof+1:end);
Kfp = KEff(presDof+1:end,1:presDof);
Kff = KEff(presDof+1:end,presDof+1:end);

% partition MEff
Mpp = MEff(1:presDof,1:presDof);
Mpf = MEff(1:presDof,presDof+1:end);
Mfp = MEff(presDof+1:end,1:presDof);
Mff = MEff(presDof+1:end,presDof+1:end);


%% Quasistatic response (Kqs*up + Mqs*ap = fiextEffp)

% get static condensate
S = -inv(Kff) * Kfp;

% get Kqs (i.e.KM) and Mqs (i.e.MM)
Kqs = Kpp + Kpf*S;
Mqs = S'*Mff*S + 2*Mpf*S + Mpp;


%% Internal dynamics ([udynf; up]=[fai;0]*eta)

% eignenfrequency (high to low) and eigenmodes (vectors)
[Fai,Omega2] = eigs(Kff,Mff,nModes+eModes,'sm');
dispM = sqrt(real(Fai'*Mff*Fai));                     % modal displacement

% normalize
for i = 1:length(dispM)
    Fai(:,i) = Fai(:,i)/dispM(i,i); 
end

% sort (low to high frequency)
[Omega,index] = sort(real(sqrt(diag(Omega2)))/2/pi); % Hz
Fai = Fai(:,index);

% filter too low frequency
[~,index] = find(Omega'>=freqTh);
Omega = Omega(index,:);
Fai = Fai(:,index);


%% Bloch analysis

% Remove the corner nodes from boundaryNodes (left,right,bottom,top)
boundaryNodes = boundaries;
boundaryNodes{1}(1) = [];
boundaryNodes{2}(1) = [];

% Bloch analysis
tb1 = clock;
[Normk, Freq, U] = blochAnalysis(K,M,boundaryNodes,W,nNodes,rDofs,extendDofs,key_cons,DirWave,Nset,eModes);

tb2 = clock;
dtb = etime(tb2,tb1); % time for Bloch analysis

figure
hold on

% first 6 modes
for iMode = 1:6
%     plot(Normk,Freq(iMode,:),'-','color',Col(iMode,:),'linewidth',4)
    patch([Normk nan],[Freq(iMode,:) nan],[Freq(iMode,:) nan],'linewidth',3,'edgecolor','interp')
end
colormap('gray')

ylim([0 Freqlim])
xlabel('\it{kL}/\pi','FontName','Times New Roman','FontWeight','Bold','FontSize',14);
ylabel('\omega [Hz]','FontName','Times New Roman','FontWeight','Bold','FontSize',14);
set(gca,'FontName','Times New Roman','FontSize',14)

close all
% plot modes on the undeformed structure
config = [1 6];
Size = [5 0 40 8];
% config = [3 2];
% Size = [5 0 12 20];


[XR,YR,Ur]=funPlotModes(key_R,Ramp,coordsRn,coordsHin,nr,r,rangeX,rangeY,U,elems,coords,extendElems,config,Size);

%% Computational homogenization

th1 = clock;
[CMO,DMO,rhoM,H,jh,NormkH, FreqH,DynParas,CS,DS,rhoS] = compHom(W,V,coords,coordsRef,presNodes,Kqs,Mqs,Mff,Mpf,S,Nset,Omega,Fai,DirWave);

th2 = clock;
dth = etime(th2,th1); % time for Bloch analysis

% compute jh2/V and H2/V
Jd = cell(1,length(Omega));
Hd = cell(1,length(Omega));
for s=1:length(Omega)
    js = jh(s,:);
    Jd{s} = [js(1)*js(1) js(1)*js(2)
             js(2)*js(1) js(2)*js(2) ]/W/W;
    Hs = H{s};
    Hd{s} = [Hs(1,1)*Hs(1,1) Hs(1,1)*Hs(2,2) Hs(1,1)*Hs(2,1) Hs(1,1)*Hs(1,2)
             Hs(2,2)*Hs(1,1) Hs(2,2)*Hs(2,2) Hs(2,2)*Hs(2,1) Hs(2,2)*Hs(1,2)
             Hs(1,2)*Hs(1,1) Hs(1,2)*Hs(2,2) Hs(1,2)*Hs(2,1) Hs(1,2)*Hs(1,2)
             Hs(2,1)*Hs(1,1) Hs(2,1)*Hs(2,2) Hs(2,1)*Hs(2,1) Hs(2,1)*Hs(1,2)]/W/W;
end

figure
hold on
for n = 1:4
% scale the imaginary part as 1/4
indkH = find(NormkH(n,:)<0);
NormkH(n,indkH) = NormkH(n,indkH)*fImg;
    
plot(NormkH(n,:),FreqH(n,:),'ko','linewidth',1,'MarkerEdgeColor','none','MarkerFaceColor','k','MarkerSize',3)

end
hold off

xlabel('\it{kL}/\pi','FontName','Times New Roman','FontWeight','Bold','FontSize',14);
ylabel('\omega [Hz]','FontName','Times New Roman','FontWeight','Bold','FontSize',14);
xlim([0 1]);
ylim([0 Freqlim])
set(gca,'FontName','Times New Roman','FontSize',14)

% dynamic parameters
Col = linspace(0,0.8,4);                             % color
Col =[Col;Col;Col]';
figure
subplot(1,2,1)
hold on
% e1e2 and e2e1 are zeros
for n = 1:2
    plot(DynParas.rho(n,:)*1e-3,DynParas.omega/2/pi,'ko','linewidth',1,'MarkerEdgeColor','none','MarkerFaceColor',Col(n,:),'MarkerSize',3)
    
end
hold off
xlabel('\rho [10^3kg/m^3]','FontName','Times New Roman','FontWeight','Bold','FontSize',14);
ylabel('\omega [Hz]','FontName','Times New Roman','FontWeight','Bold','FontSize',14);
xlim([-1 1]*rhoR(1)*1e-3)
ylim([0 Freqlim])
set(gca,'FontName','Times New Roman','FontSize',14)
legend('e_1e_1','e_2e_2','e_1e_2','e_2e_1')
subplot(1,2,2)
hold on
for n = 1:2
    plot(DynParas.C(n,:)*1e-9,DynParas.omega/2/pi,'ko','linewidth',1,'MarkerEdgeColor','none','MarkerFaceColor',Col(n,:),'MarkerSize',3)
    
end
hold off
xlabel('C [GPa]','FontName','Times New Roman','FontWeight','Bold','FontSize',14);
xlim([-5 5]*E*1e-9)
ylim([0 Freqlim])
set(gca,'FontName','Times New Roman','FontSize',14)
set(gcf,'Units','centimeters','Position',[5 5 16 14])

% save(MAT_dynamic_parameters,'DynParas');

%% Dispersion comparison

figure
hold on

% dispersion curves by Bloch analysis
sModes = 6;      % compare the first 6 modes
for iMode = 1:sModes
    plot(Normk,Freq(iMode,:),'b-','linewidth',3)
%     patch([Normk nan],[Freq(iMode,:) nan],[Freq(iMode,:) nan],'linewidth',3,'edgecolor','interp')
end
% colormap('cool')

% dispersion curves by homogenization model
nn = 2;
for n = 1:nn:4
plot(NormkH(n,:),FreqH(n,:),'ro','linewidth',1,'MarkerEdgeColor','none','MarkerFaceColor','r','MarkerSize',3)

end

for n = 2:nn:4
plot(NormkH(n,:),FreqH(n,:),'ro','linewidth',1,'MarkerEdgeColor','none','MarkerFaceColor','r','MarkerSize',3)

end

hold off
xlabel('\it{kL}/\pi','FontName','Times New Roman','FontWeight','Bold','FontSize',14);
ylabel('\omega [Hz]','FontName','Times New Roman','FontWeight','Bold','FontSize',14);
set(gca,'FontName','Times New Roman','FontSize',14)
% 
title('Dispersion curves','FontName','Times New Roman','FontWeight','Bold','FontSize',14)
ylim([0 Freqlim])
grid

switch RANGE
case '0 to 1'
xlim([0 1])
set(gcf,'Units','centimeters','Position',[5 5 8.5 14])

case '-1 to 1'
axis('square')
xlim([-1*fImg 1])
set(gcf,'Units','centimeters','Position',[5 5 16 14])
end

% save(MAT_dispersion_comparison,'Normk','Freq','NormkH','FreqH');


