%-------------------------------------------------------------------------%
% Background
% Main function for the macroscopic analysis for a domain filled with 
% metamaterial partially
% Dispersion analysis remains as a convenient reference
% Created by Lei, TU/e, 2017

% Notes
% Model: Plane stress of Plane strain (linear elasticity)
% Element type: 4-node element
% Unit system (unit thickness): N,Pa,m,kg;
% sparse matrix

% Key input
% data from microscopic analysis
% double negative system (case D) is focused on

%-------------------------------------------------------------------------%
clear,clc

%% Material Parameters

% M matrix
Paras.M.E = 25e9;                         % modulus
Paras.M.mu = 0.3;                         % poisson ratio
Paras.M.rho = 2e3;                        % density
rhoMat = Paras.M.rho * (1 - pi*(0.25)^2);
rhoMat = rhoMat*eye(2);
Paras.M.W = 20e-3;                        % width

Paras.M.mesh = 'Matrix2D_Q2.txt';         % mesh

Paras.M.Model = 'Planestrain';            % stress state

% R resonator
Paras.R.nr = 3;                           % resonator number
Paras.R.keyR = 'D';                       % type
Paras.R.keyC = 0;                         % 1: add extra constriants on the nodal dispalcements

Paras.R.coordsRef = [0.5 0.5];            % normalized reference coordinates
Paras.R.r = 0.05;                         % normalized radius

alpha = 15/180*pi;                   % reference 21.8deg
shHin = tan(alpha)*0.25;
Paras.R.coordsHin = [0.5 0.5+shHin;                 % hinge joint position
                    0.5 0.5-shHin];
Paras.R.coordsRn = [0.5 0.70;
                    0.5 0.30;
                    0.5 0.50];            % normalized resonator positions
                                          
                                          
KR = 0.005 * [1 1e-8];                     % normalized spring stiffness per unit thickness at horizontal state
Paras.R.KR = [KR 0.5*KR]*Paras.M.E;       % extra springs for type D
Paras.R.rhoR = 50 * [1 1]*Paras.M.rho;    % density

% vibration analysis
Paras.VIB.dw = [1 0];                     % wave vector
Paras.VIB.fr = 0.5e4;                    % display range for frequency
Paras.VIB.kr = '0 to 1';                  % display range for wave number
Paras.VIB.config = [1 6];                 % settings for mode plotting [3 2],[5 0 12 20]; [1 6], [5 0 40 8]
Paras.VIB.Size = [5 0 40 8];

nModes = 6;                               % mode number
eModes = 2;                               % mode number to be removed
nSamples = 100;                           % for Bloch analysis
Nomega = 500*nSamples;                    % for homogenization model 
freqTh = 100;                             % frequency threshold
Ramp = 0;                                 % display amplitude

Paras.VIB.Nset = [nModes nSamples Nomega freqTh Ramp eModes];

%% Settings for macro analysis

% input
% example value

% positive refraction
% double pass: 1300, 1800
% primary pass: 3600 
% shear pass: 2280

% negative refraction: 2985, 3000 (0.5)

freqE = 0.2985e4;                          % excitation frequency, Hz
Refraction = 'Negative';

% MAT_macro_configuration = ['Macro_configuration'];
% MAT_refraction =['Negative_refraction'];
aviWave = VideoWriter('negRef_2985Hz_S.avi');

OmegaE = freqE * 2*pi;                  
fE = 1e5;                                % excitation force amplitude
% Af = 0.5;                                  % to highlight the wave
intS = [1:3];
intP = [1:3];

% 0: full single boundary is used, 1: the same size is equal or smaller than the prism edge
key_source = 1; 

% mesh
We = 3*Paras.M.W;                     % element width, uniform mesh

nElemX = 100; nElemY = 200;              % element number
nElem = nElemX * nElemY;

nNodeX = nElemX+1; nNodeY = nElemY+1;    % node number
nNode = nNodeX * nNodeY;

nNE = 4;                                 % 4-node element

% get coordinates
% |2*nNodeX+1...   |
% |nNodeX+1...     |
% |1 2 3 4...nNodeX|

coords = zeros(nNode,2);

Xt = We * nElemX;                        % Range X and Y
Yt = We * nElemY;
X0 = 0;                                  % reference coordinates
Y0 = -Yt*0.3;

Ybound = (Y0 + [-Y0 Yt*0.675])*1000;      % snapshotted zone, reference: 0.675 and 0.375
Xbound = (X0 + [0 Xt*0.75])*1000;


for iY = 1:nNodeY
    for iX = 1:nNodeX
        coords(iX + (iY-1)*nNodeX, 1) = (iX-1) * We + X0;
        coords(iX + (iY-1)*nNodeX, 2) = (iY-1) * We + Y0;
        
    end
end

% connectivity
% |nElemX+1...     |
% |nElemX+1...     |
% |1 2 3 4...nElemX|

% anti-clockwise
% |4 3|
% |1 2|

elems = zeros(nElem,nNE);

for iY = 1:nElemY
    for iX = 1:nElemX
        elems(iX + (iY-1)*nElemX,:) = [iX+nNodeX*(iY-1)
                                       iX+nNodeX*(iY-1)+1
                                       iX+nNodeX*iY+1
                                       iX+nNodeX*iY     ]';
        
    end
end

FlagElem = zeros(1,nElem);  % intersted element zone
for iElem = 1:nElem
cenElem = mean(coords(elems(iElem,:),:));
    if (cenElem(1)>=Xbound(1)/1000 && cenElem(1)<=Xbound(2)/1000) && (cenElem(2)>=Ybound(1)/1000 && cenElem(2)<=Ybound(2)/1000)
        FlagElem(iElem) = 1;
    end
end    
% material allocation
% metamaterial is located in a triangular zone, 3 points
% |3 2|
% |1  |

MatNode = zeros(nNode,1);                     % 1: metamaterial; 0: normal
Rcoords = [(coords(:,1)-X0)/Xt (coords(:,2)-Y0)/Xt]; % normalized nodal coordinates, about Xt

% metamaterial zone
alpha1 = pi/6;            % incident angle
kT = 1/tan(pi/6);
Rcoords1 = [0.01 0.74];
Rcoords2 = 0.3*[1 kT] + Rcoords1;
Rcoords3 = 0.3*[0 kT] + Rcoords1;
metacoords = [Rcoords1;Rcoords2;Rcoords3;Rcoords1]*Xt + [1;1;1;1]*[X0 Y0];

for iNode = 1:nNode
    if (Rcoords(iNode,1)>Rcoords1(1)) && (Rcoords(iNode,2)<Rcoords3(2))...
        && (kT*(Rcoords(iNode,1)-Rcoords1(1))+Rcoords1(2)<Rcoords(iNode,2))
    
    MatNode(iNode) = 1;
        
    end    
end

figure
hold on
% normZone
normZone =[Xbound(1) Ybound(1); Xbound(2) Ybound(1); Xbound(2) Ybound(2); Xbound(1) Ybound(2);];
fill(normZone(:,1),normZone(:,2),'b','EdgeColor','none','facealpha',1);

% prism
plot(metacoords(:,1)*1000,metacoords(:,2)*1000,'-','color',[0.5 0.5 0.5],'linewidth',3)
patch(metacoords(:,1)*1000,metacoords(:,2)*1000,'m','EdgeColor','none','facealpha',1)

hold off
axis('tight')
axis('equal')
axis off
xlabel('X [mm]','FontName','Times New Roman','FontWeight','Bold','FontSize',14);
ylabel('Y [mm]','FontName','Times New Roman','FontWeight','Bold','FontSize',14);
set(gca,'FontName','Times New Roman','FontSize',14)

Size = [10 0 15 15];

set(gcf,'Units','centimeters','Position',Size)
% title('Mesh','FontName','Times New Roman','FontWeight','Bold','FontSize',14)
ylim(Ybound)
xlim(Xbound)

% save(MAT_macro_configuration,'normZone','metacoords');

% define a zone to evaluate the maximum and minimum of the refraction
% wave field
EvaNode = zeros(nNode,1); 
Eshift1 = 0.01;        % reltaive to the refraction surface
% Eshift2 = 0.11;

Ecoords1 = (Rcoords1 + Rcoords2)/2 + Eshift1*[cos(pi/6) -sin(pi/6)];
% Ecoords2 = (Rcoords1 + Rcoords2)/2 + Eshift2*[cos(pi/6) -sin(pi/6)];

for iNode = 1:nNode
    if Rcoords(iNode,1)>Ecoords1(1)&&Rcoords(iNode,2)<Ecoords1(2);
    
    EvaNode(iNode) = 1;
        
    end    
end

evacoords = [Ecoords1;]*Xt + [1]*[X0 Y0];


%% ERVE properties

[CMT,rhoMT,rhoS,CS,DS,rhoM,CMO,DMO,MR,MM,H,jh,Omega,Fai,DynParas] = funRRVEprops(Paras);

t1 = clock;
% form coupling term pairs
% for case D, actually 3 effective terms, [eta1; eta2; eta3] along X X and Y
% incert a fictive coupling term after eta1 along Y.

st = 1e-7;
np = 2;                                  % two pairs of etas
Hs = cell(1,4);
Hs{1} = H{1}; Hs{2} = st*ones(2,2); Hs{3} = H{2}; Hs{4} = H{3};

js = zeros(4,2);
js(1,:) = jh(1,:); js(2,:) = st*[1 1]; js(3,:) = jh(2,:); js(4,:) = jh(3,:);

OmegaR = [Omega(1) st Omega(2) Omega(3)]*2*pi;% resonant frequency

Hp = cell(1,np);
jp = cell(1,np);
Omegap = cell(1,np);

for i = 1:np
    id = 2*i-1;
    Hp{i} = [Hs{id}(1,1) Hs{id}(2,2) Hs{id}(1,2) Hs{id}(2,1);
             Hs{id+1}(1,1) Hs{id+1}(2,2) Hs{id+1}(1,2) Hs{id+1}(2,1)]';
    jp{i} = [js(id,:);js(id+1,:)]';
    Omegap{i} = [OmegaR(id) 0; 0 OmegaR(id+1)];
end     

close all

%% Macro FE model

% quasi-static properties of the metamaterial
CMM = CMO;
RhoMM = rhoM;

% properties of the normal material, CMT, RhoMT
RhoMT = rhoMT;

totDofs = [1:2*nNode]';     % without coupling
MDof = length(totDofs);                 % matrix DOF
RDof = 2*MDof;                            % resonator of eta DOF

% assemble system matrices
order = 0;
coordsElem = coords(elems(1,:),:);
[nElemNode,nIntElem,dVElem] = getElemProp(coordsElem,order);

% define sparse assembly vectors
nElemDof = 2*nElemNode;

rowV = zeros(nElemDof^2,nElem);
colV = zeros(nElemDof^2,nElem);

KMV = zeros(nElemDof^2,nElem);
MMV = zeros(nElemDof^2,nElem);

KRV = zeros(np*nElemDof^2,np*nElem);
MRV = zeros(np*nElemDof^2,np*nElem);

KCV = zeros(nElemDof^2,np*nElem);
MCV = zeros(nElemDof^2,np*nElem);

for iElem = 1:nElem
    
    elemNode = elems(iElem,:);
    elemDofs = getDofs(elemNode);
    nElemDof = length(elemDofs);
    
    % shape function information
    sData = getElemShapeData(coords(elemNode,:),order);
    
    % initialize (2 pairs of etas)
    KM = zeros(nElemDof);                % stiffness matrix for matrix
    MM = zeros(nElemDof);                % mass matrix for matrix
    
    KR = zeros(np*nElemDof);              % resonator stiffness
    MR = zeros(np*nElemDof);              % resonator mass
    
    KC = zeros(nElemDof,np*nElemDof);     % stiffness coupling 
    MC = zeros(nElemDof,np*nElemDof);     % mass coupling
    
    % write element matrices
    for is =1:length(sData)
        iData = sData{is};
        [B,H]= getBH(iData.dhdx,iData.h);
        
        % judge metamaterial or normal material
        if MatNode(elemNode(is))==1
            C = CMM;
            Rho = RhoMM;
        else
            C = CMT;
            Rho = RhoMT;
        
        end   
        
        KM = KM + B'* C * B * iData.weight * dVElem;
        MM = MM + H'* Rho * H * iData.weight * dVElem;
        
        for i = 1:np
            index = (i-1)*nElemDof +1 : i*nElemDof;
            
            % coupling terms
            KC(:,index) = KC(:,index) + B'* Hp{i} * H * iData.weight * dVElem;
            MC(:,index) = MC(:,index) + H'* jp{i} * H * iData.weight * dVElem;
            
            % resonator terms
            KR(index,index) = KR(index,index) + H' * (Omegap{i}.^2) * H * iData.weight * dVElem;
            MR(index,index) = MR(index,index) + H' * H * iData.weight * dVElem; 
            
        end
        
    end
    
    % element row and column matrices
    rowID = elemDofs * ones(1,nElemDof);
    colID = ones(nElemDof,1) * elemDofs';
    
    % form sparse assembly vectors
    rowV(:,iElem) = rowID(:);
    colV(:,iElem) = colID(:);
    
    KMV(:,iElem) = KM(:);
    MMV(:,iElem) = MM(:);
    
    for i = 1:np
        index1 = (i-1)*nElemDof +1 : i*nElemDof;
        index2 = (i-1)*nElemDof^2 +1 : i*nElemDof^2;
        
        kr = KR(index1,index1);
        mr = MR(index1,index1);
        
        kc = KC(:,index1);
        mc = MC(:,index1);
        
        KRV(index2,iElem+(i-1)*nElem) = kr(:);
        MRV(index2,iElem+(i-1)*nElem) = mr(:);
        
        KCV(:,iElem+(i-1)*nElem) = kc(:);
        MCV(:,iElem+(i-1)*nElem) = mc(:);
    
    end 
    
end

%% Form sparse matrices
KMsp = sparse(rowV,colV,KMV);
MMsp = sparse(rowV,colV,MMV);

KR1 = KRV(1:nElemDof^2,1:nElem);
KR2 = KRV((1:nElemDof^2)+nElemDof^2,(1:nElem)+nElem);

MR1 = MRV(1:nElemDof^2,1:nElem);
MR2 = MRV((1:nElemDof^2)+nElemDof^2,(1:nElem)+nElem);

KRsp = [sparse(rowV,colV,KR1(:)) sparse(MDof,MDof)
        sparse(MDof,MDof)        sparse(rowV,colV,KR2(:))];
    
MRsp = [sparse(rowV,colV,MR1(:)) sparse(MDof,MDof)
        sparse(MDof,MDof)        sparse(rowV,colV,MR2(:))];

KC1 = KCV(:,1:nElem);
KC2 = KCV(:,(1:nElem)+nElem);

MC1 = MCV(:,1:nElem);
MC2 = MCV(:,(1:nElem)+nElem);

KCsp = [sparse(rowV,colV,KC1(:)) sparse(rowV,colV,KC2(:))];
MCsp = [sparse(rowV,colV,MC1(:)) sparse(rowV,colV,MC2(:))];

% assemble final system matrices
Vc = 1/Paras.M.W/Paras.M.W;

Ksys = [KMsp   -OmegaE^2*Vc*KCsp;
        -OmegaE^2*KCsp'  KRsp   ];
    
Msys = [MMsp   Vc*MCsp;
        MCsp'  MRsp   ];

%  Rayleigh damping
c1 = 0;
c2 = 0;
Csys = c1*Ksys + c2*Msys;

%% Boundary conditions
LNodes = 1:nNodeX:nNode;          % left-right-bottom-top
RNodes = nNodeX:nNodeX:nNode;

BNodes = 1:nNodeX;
TNodes = (nNode-nNodeX+1):nNode;

UDofs = [1:2:MDof; 2:2:MDof]';     % [Ux Uy Eta1x Eta1y Eta2x Eta2y]
EtaDofs = [1+MDof:2:2*MDof; 2+MDof:2:2*MDof]';
EtaDofs = [EtaDofs [1+2*MDof:2:3*MDof; 2+2*MDof:2:3*MDof]'];
Dofs = [UDofs EtaDofs];

LDofs = UDofs(LNodes,:);
RDofs = UDofs(RNodes,:);
BDofs = UDofs(BNodes,:);
TDofs = UDofs(TNodes,:);

% constrainted or removed Dofs
ConsEtaDofs = [Dofs(MatNode==0,3);Dofs(:,4);Dofs(MatNode==0,5);Dofs(MatNode==0,6)];
% ConsEtaDofs = [Dofs(:,3);Dofs(:,4);Dofs(:,5);Dofs(:,6)];
ConsDofs = ConsEtaDofs;

% prescribed DOFs (default: left boundary)
PresNodeID = [1:length(LNodes)]';
if key_source == 1
   Sshift = [0.21 0.21] -0.1*ones(1,2);         % shift size, lower, higher, reference: [0.19 0.23]
   Source = [ 0 (Rcoords1(2)+Sshift(1))*Xt+Y0;
              0 (Rcoords2(2)-Sshift(2))*Xt+Y0];                   % relative coordinates of the source boundary

   Lcoords = coords(LNodes,:);
   PresNodeID = find(Source(1,2)<=Lcoords(:,2) & Lcoords(:,2)<=Source(2,2));
% PresNodeID = [PresNodeID(1)-1; PresNodeID; PresNodeID(end)+1];

   PresDofs = LDofs(PresNodeID,1);       
else
   PresDofs = LDofs(:,1); 
end

FreeDofs = setdiff([1:3*MDof]',ConsDofs);
nFreeDof = length(FreeDofs);

[~,PresIndex] = ismember(PresDofs,FreeDofs);

% form the viscous term
BoundDofs = [LDofs;RDofs;BDofs;TDofs];
nBound = size(BoundDofs,1);

for ic = 1:2
    for jc = 1:2
    Csys(BoundDofs(:,ic),BoundDofs(:,jc)) = Csys(BoundDofs(:,ic),BoundDofs(:,jc)) + eye(nBound) * sqrt(CS(ic,jc)*rhoMat(ic,jc))*We/2;
    end
end

% apply teh force
F = zeros(nFreeDof,1);
F(PresIndex) = fE;

%% Reduce matrices
Kff = Ksys(FreeDofs,FreeDofs);
Mff = Msys(FreeDofs,FreeDofs);
Cff = Csys(FreeDofs,FreeDofs);

Kdyn = Kff + 1i*OmegaE*Cff - OmegaE^2*Mff;

%% solve equations at steady state (F=F0exp(iw),U=U0exp(iw))
Disp = zeros(3*MDof,1);
Disp(FreeDofs) = Kdyn\F;


t2 = clock;
dt = etime(t2,t1);


%% Displacement field

% omit the displacement field inside the metamaterial zone
meNode = find(MatNode==1);
Disp([2*meNode-1;2*meNode]) = 0;

% indentify the nodes in the evaluation zone
evNode = find(EvaNode==1);

Ux = real(Disp(1:2:MDof));
Uy = real(Disp(2:2:MDof));

U = sqrt(Ux.^2 + Uy.^2);
maxU = max(U(evNode));    % 2985Hz: 5.6108e-6m, 3600Hz: 4.5587e-6m
% maxU = 5.6e-6;

% absolute displacement
figure
hold on
for iElem = 1:nElem
    if FlagElem(iElem) == 1
    elemNode = elems(iElem,:);
    nodeZone = [elemNode elemNode(1)];
         fill(coords(nodeZone,1)*1000,coords(nodeZone,2)*1000,U(nodeZone),'EdgeColor','none');
    end
    
end
plot(coords(LNodes(PresNodeID),1)*1000,coords(LNodes(PresNodeID),2)*1000,'r-','linewidth',4)

% prism
plot(metacoords(:,1)*1000,metacoords(:,2)*1000,'-','color',[0.25 0.25 0.25],'linewidth',3)
patch(metacoords(:,1)*1000,metacoords(:,2)*1000,[0.5 0.5 0.5],'EdgeColor','none','facealpha',1)

LPresNodeID = LNodes(PresNodeID);

% % direction vector
% S = 1;
% drUx = Ux./U;              % normalized
% drUy = Uy./U;
% selNodes = 1:5:nNode;
% quiver(coords(selNodes,1)*1000,coords(selNodes,2)*1000,drUx(selNodes),drUy(selNodes),S,'k-','linewidth',1);

hold off
axis('tight')
axis('equal')
set(gca,'xtick',[],'xcolor','k','ytick',[],'ycolor','k','linewidth',2);
set(gca,'FontName','Times New Roman','FontSize',16,'color','k')

% xlabel('X [mm]','FontName','Times New Roman','FontWeight','Bold','FontSize',14);
% ylabel('Y [mm]','FontName','Times New Roman','FontWeight','Bold','FontSize',14);
set(gca,'FontName','Times New Roman','FontSize',14)
colorbar off
colormap('jet')
caxis([0 maxU]);
% title('Mixed waves','FontName','Times New Roman','FontWeight','Bold','FontSize',14,'interpreter','latex')
box on
ylim(Ybound)
xlim(Xbound)

set(gcf,'Units','centimeters','Position',Size)

hbar = colorbar;
% Sizebar = [0.88 0.107 0.03 0.819];
Sizebar = [0.94 0.126 0.03 0.783];
set(hbar, 'Position', Sizebar);
set(hbar, 'ytick', [ ], 'yticklabel',[ ]);


%% Separate the longitudinal and transverse waves
% push the dispalcemnets on the uniform grids
X = (0:1:nElemX)*We + X0;
Y = (0:1:nElemY)*We + Y0;

% indentify the nodes in the evaluation zone
EvaX = find(X>evacoords(1));
EvaY = find(Y<evacoords(2));

DispX = zeros(length(Y),length(X));
DispY = zeros(length(Y),length(X));

for iY = 1:nNodeY
    for iX = 1:nNodeX
        iNode = iX + (iY-1)*nNodeX;
        DispX(iY,iX) = Disp(2*iNode-1);
        DispY(iY,iX) = Disp(2*iNode);
          
    end
end

% get partial derivations
Uxx = zeros(length(Y),length(X));
Uxy = zeros(length(Y),length(X));
Uyx = zeros(length(Y),length(X));
Uyy = zeros(length(Y),length(X));

Uxx(:,2:end) = (DispX(:,2:end) - DispX(:,1:end-1))/We;
Uxy(2:end,:) = (DispX(2:end,:) - DispX(1:end-1,:))/We;

Uyx(:,2:end) = (DispY(:,2:end) - DispY(:,1:end-1))/We;
Uyy(2:end,:) = (DispY(2:end,:) - DispY(1:end-1,:))/We;

S0 = Uxy - Uyx;                     % complex solution
P0 = Uxx + Uyy;

% phaseC = exp(-1i*pi/2);             % pi/2 phase correction
S = real(S0);                % transverse wave
P = real(P0);                % longitudinal wave

maxS = max(max(S(EvaY,EvaX))); minS = min(min(S(EvaY,EvaX)));
maxP = max(max(P(EvaY,EvaX))); minP = min(min(P(EvaY,EvaX)));

maxPS = max(abs([maxS minS maxP minP]));  % 2985Hz: 4.8917e-5, 3600Hz: 3.4266e-5
% maxPS = 4.9e-5;

% normalize
DispN = sqrt(DispX.^2 + DispY.^2);
% S = S/max(max(S(:,intS)));            
% P = P/max(max(P(:,intP)));

nLevels = 250;
RX = 1:round(nNodeX/4*3);
% RY = round(nNodeY/4):round(nNodeY/4*3);
RY = round(nNodeY/4):round(nNodeY/4*3);

figure
hold on
contourf(X(RX)*1000,Y(RY)*1000,P(RY,RX),nLevels,'linestyle','none');

% source
plot(coords(LNodes(PresNodeID),1)*1000,coords(LNodes(PresNodeID),2)*1000,'r-','linewidth',4)

% prism
plot(metacoords(:,1)*1000,metacoords(:,2)*1000,'-','color',[0.25 0.25 0.25],'linewidth',3)
patch(metacoords(:,1)*1000,metacoords(:,2)*1000,[0.5 0.5 0.5],'EdgeColor','none','facealpha',1)

colormap('jet')
caxis([-maxPS maxPS]);
axis('tight')
axis('equal')
set(gca,'xtick',[],'xcolor','k','ytick',[],'ycolor','k','linewidth',2);
set(gca,'FontName','Times New Roman','FontSize',16,'color','k')
colorbar off
% title('P wave','FontName','Times New Roman','FontWeight','Bold','FontSize',14,'interpreter','latex')
box on
ylim(Ybound)
xlim(Xbound)

set(gcf,'Units','centimeters','Position',Size)

figure
hold on
contourf(X(RX)*1000,Y(RY)*1000,S(RY,RX),nLevels,'linestyle','none');

% source
plot(coords(LNodes(PresNodeID),1)*1000,coords(LNodes(PresNodeID),2)*1000,'r-','linewidth',4)

% prism
plot(metacoords(:,1)*1000,metacoords(:,2)*1000,'-','color',[0.25 0.25 0.25],'linewidth',3)
patch(metacoords(:,1)*1000,metacoords(:,2)*1000,[0.5 0.5 0.5],'EdgeColor','none','facealpha',1)

colormap('jet')
caxis([-maxPS maxPS]);
axis('tight')
axis('equal')
set(gca,'xtick',[],'xcolor','k','ytick',[],'ycolor','k','linewidth',2);
set(gca,'FontName','Times New Roman','FontSize',16,'color','k')
colorbar off
% title('S wave','FontName','Times New Roman','FontWeight','Bold','FontSize',14,'interpreter','latex')
box on
ylim(Ybound)
xlim(Xbound)

set(gcf,'Units','centimeters','Position',Size)
hbar = colorbar;
set(hbar, 'Position', Sizebar);
set(hbar, 'ytick', [ ], 'yticklabel',[ ]);

% set(hbar, 'FontSize', 14);
% set(hbar, 'ytick', [0 Af], 'yticklabel',[0 1]);
% set(hbar, 'Position', [0.91 0.110 0.015 0.815]);

% save(MAT_refraction,'freqE','FlagElem','coords','elems','U','LPresNodeID','metacoords','X','Y','P','S');

% close all

%% Required plot
% figure
% 
% hold on
% contourf(X(RX)*1000,Y(RY)*1000,S(RY,RX),nLevels,'linestyle','none');
% % source
% plot(coords(LNodes(PresNodeID),1)*1000,coords(LNodes(PresNodeID),2)*1000,'r-','linewidth',4)
% 
% % prism
% plot(metacoords(:,1)*1000,metacoords(:,2)*1000,'-','color',[0.5 0.5 0.5],'linewidth',3)
% patch(metacoords(:,1)*1000,metacoords(:,2)*1000,'m','EdgeColor','none','facealpha',1)
% 
% % incident diretcion
% annotation('arrow',[0.23 0.41],[0.517 0.517],'LineStyle','-','linewidth',4,'color','b','HeadStyle','plain','headwidth',8,'headlength',12)
% colormap('jet')
% % caxis([-Af Af]);
% axis('tight')
% axis('equal')
% set(gca,'FontName','Times New Roman','FontSize',16)
% colorbar off
% title('Shear wave','FontName','Times New Roman','FontWeight','Bold','FontSize',16,'interpreter','latex')
% box on
% ylim(Ybound)
% xlim(Xbound)
% set(gca,'xtick',[])
% set(gca,'ytick',[])
% 
% set(gcf,'Units','centimeters','Position',Size)
% hbar = colorbar;
% set(hbar, 'ytick', [-Af Af], 'yticklabel',[-1 1]);
% set(hbar, 'Position', Sizebar);
% set(gcf,'Units','centimeters','Position',[5 0 10 16])

%% Create a movie
T = 2*pi / OmegaE;           % period
t = linspace(0,2*T,50);

MOV = moviein(length(t));

aviWave.FrameRate = round(length(t)/2);                   
aviWave.Quality = 100;
open(aviWave);

fig = figure;
% exp(i*t(im)*OmegaE/2*pi);

switch Refraction
    case 'Positive'
        W0 = P0;        % interested wave component
        Wtitle = 'P';
    case 'Negative'
        W0 = S0;
        Wtitle = 'S';
end


% LimW0 = [-max(max(abs(W0))) max(max(abs(W0)))];
for im = 1:length(t)
    clf;
    hold on
    tC = t(im)*1000;
    tName = num2str(round(tC,4));
    Field = real(W0(RY,RX)*exp(1i*t(im)*OmegaE));     % real part
    
    contourf(X(RX)*1000,Y(RY)*1000,Field,nLevels,'linestyle','none');
    % source
    plot(coords(LNodes(PresNodeID),1)*1000,coords(LNodes(PresNodeID),2)*1000,'r-','linewidth',4)

    % prism
    plot(metacoords(:,1)*1000,metacoords(:,2)*1000,'-','color',[0.25 0.25 0.25],'linewidth',3)
    patch(metacoords(:,1)*1000,metacoords(:,2)*1000,[0.5 0.5 0.5],'EdgeColor','none','facealpha',1)

    colormap('jet')
    caxis([-maxPS maxPS]);
    axis('tight')
    axis('equal')
    set(gca,'FontName','Times New Roman','FontSize',16,'color','k')
    colorbar off
    title([Wtitle '-polarization (' '$t$=' tName 'ms)'],'FontName','Times New Roman','FontWeight','Bold','FontSize',16,'interpreter','latex')
    box on
    ylim(Ybound)
    xlim(Xbound)
    set(gca,'xtick',[],'xcolor','k','ytick',[],'ycolor','k','linewidth',2);
    set(gcf,'Units','centimeters','Position',Size)
    drawnow;
    
    % write current figure into the movie
    MOV(:,im) = getframe(fig);
    writeVideo(aviWave,MOV(:,im));
    
end

close(aviWave);


%% Refraction angle (based on direct estimation on wave field with FFT)

% dynamic effective properties of the metamaterial
rhoD = [interp1(DynParas.omega,DynParas.rho(1,:),OmegaE) interp1(DynParas.omega,DynParas.rho(3,:),OmegaE);
        interp1(DynParas.omega,DynParas.rho(4,:),OmegaE) interp1(DynParas.omega,DynParas.rho(2,:),OmegaE)];

CD = [interp1(DynParas.omega,DynParas.C(1,:),OmegaE) interp1(DynParas.omega,DynParas.C(3,:),OmegaE);
      interp1(DynParas.omega,DynParas.C(4,:),OmegaE) interp1(DynParas.omega,DynParas.C(2,:),OmegaE)];

c1P = sqrt(CD(1,1)/rhoD(1,1)); % phase velocity       
c1S = sqrt(CD(2,2)/rhoD(2,2));

c2P = sqrt(CMT(1,1)/rhoMT(1,1)); % phase velocity in material 2, normal
c2S = sqrt(CMT(4,4)/rhoMT(2,2));

% P wave
alpha2P.P = asin(sin(alpha1)/c1P*c2P) * 180/pi;  % P-P
alpha2P.S = asin(sin(alpha1)/c1S*c2P) * 180/pi;  % S-P

% transverse wave
alpha2S.P = asin(sin(alpha1)/c1P*c2S) * 180/pi;
alpha2S.S = asin(sin(alpha1)/c1S*c2S) * 180/pi;

% % plot a refrence vector
% VX = [0 1000];
% K = -tan((30-10.58)*pi/180);
% VY = [2250 2250+K*1000];
% plot(VX,VY,'k-','linewidth',4);
%     
% 
% plot([3000 3000],[0 4500],'k-','linewidth',4);



