%-------------------------------------------------------------------------%
% Background
% Function to get properties of enriched RVE
% Voigt notation is used
% Created by Lei, TU/e, 2017
%-------------------------------------------------------------------------%
function [CMT,rhoMT,rhoS,CS,DS,rhoM,CMO,DMO,MR,MM,H,jh,Omega,Fai,DynParas] = funRRVEprops(Paras)

%% settings

% matrix
E = Paras.M.E;                     % modulus
mu = Paras.M.mu;                   % possion ratio
rho = Paras.M.rho;                 % mass density
W = Paras.M.W;                     % width

meshfile = Paras.M.mesh;           % RVE mesh

Kc = E/3/(1-2*mu);
Gc = E/2/(1+mu);

Model = Paras.M.Model;             % stress state
C = planeModel(Kc,Gc,Model);       % tangent

% output the matrix properties, standard forms
CMT = zeros(4,4);
CMT(1:3,1:3) = C;
CMT(3:4,3:4) = C(3,3)*ones(2,2);

rhoMT = zeros(2,2);
rhoMT(1,1) = rho;
rhoMT(2,2) = rho;

% resonator
nr = Paras.R.nr;                   % number of resonators 
key_R = Paras.R.keyR;              % select the resonator configuration
key_cons = Paras.R.keyC;           % 1: add extra constriants on the nodal dispalcements

coordsRef = W * Paras.R.coordsRef; % reference postion, default at the center
r = W * Paras.R.r;                 % radius
rs = 3/3 * r;                      % rotation radius of the hinge joint, case C
coordsHin = W * Paras.R.coordsHin; % hinge joint positions, case B and D
coordsRn = W * Paras.R.coordsRn;   % resonator positions, one for cases A and C, two for B, three for D

KR = Paras.R.KR;                   % spring stiffness
rhoR = Paras.R.rhoR;               % mass density

% vibration analysis
DirWave = Paras.VIB.dw;            % wave vetcor
Freqlim = Paras.VIB.fr;               % display range for frequency
RANGE = Paras.VIB.kr;              % display range for wave number
config = Paras.VIB.config;          % settings for mode plotting
Size = Paras.VIB.Size;

Nset = Paras.VIB.Nset;             % analysis settings
nModes = Nset(1);
eModes = Nset(end);
Ramp = Nset(end-1);
freqTh = Nset(end-2);


%% Mass
MM = rho*(W^2 - pi*(W*0.25)^2);          % matrix mass
MR = rhoR * (pi*r^2);              % resonator mass

%% Read the mesh and constraints

[coords,elems,presNodes,boundaries] = readMeshMicro(meshfile);
coords = coords * W;

nNodes = length(coords);               % total node number
totDof = 2*nNodes;                     % total DOFs

nElems = length(elems);                % total element number
nElemNodes = length(elems(1,:));

presDofs = getDofs(presNodes);         % prescribed DOFs       
presDof = length(presDofs);

%% Approximate volume

% X-Y sizes
maxcoordsX = max(coords(:,1));
mincoordsX = min(coords(:,1));
maxcoordsY = max(coords(:,2));
mincoordsY = min(coords(:,2));
rangeX = maxcoordsX - mincoordsX;
rangeY = maxcoordsY - mincoordsY;

V = rangeX*rangeY;

%% Select integration scheme

coordsElem = coords(elems(1,:),:);     % element sample
order = 0;                             % default value (1,2 are optional)

% element properties
[nElemNodes,nIntElem,dVElem] = getElemProp(coordsElem,order);
totInt = nIntElem*nElems;              % total integration points

%% Collect tangent stiffness

CInt = cell(1,totInt);
for iInt =1:totInt
    CInt{iInt} = C;
end

%% Assemble system stiffness and mass matrice

% matrix stiffness and mass K*u + M*a = fext,a: acceleration
[K,M] = assembleKeMe(coords,elems,CInt,rho,nIntElem,dVElem,order);

e = 0.05;
% coupNodes = [24 20 18 22];   % for Q0 structure

switch key_R
case 'A'
% get coupling nodes (horizontal and vertical)
CoupX = min(coords(:,1))+ rangeX*0.5;
CoupY = min(coords(:,2))+ rangeY/2;

IndexX = find((1-e) * CoupY <= coords(:,2) & coords(:,2)<= (1+e) * CoupY );
IndexY = find((1-e) * CoupX <= coords(:,1) & coords(:,1)<= (1+e) * CoupX );
coupNodes = [IndexX(2) IndexX(1) IndexY(1) IndexY(2)]; % left right bottom top

% add a cruciform resonator
[K,M,rDofs,extendElems,extendDofs] = addResonatorA(K,M,KR,MR,coords,coupNodes,coordsRn(1,:));

case 'B'
%  get coupling nodes (horizontal)
CoupY = min(coords(:,2))+ rangeY*0.5;
IndexX = find((1-e) * CoupY <= coords(:,2) & coords(:,2)<= (1+e) * CoupY );
coupNodes = [IndexX(2) IndexX(1)];   % left right

% add a herringboned resonator
[K,M,rDofs,extendElems,extendDofs,Alpha] = addResonatorB(K,M,KR,MR,nr,coords,coupNodes,coordsHin);

case 'C'
% get coupling nodes (horizontal and vertical)
% R = 0.25*W;
% Rr = sqrt((coords(:,1)-coordsRef(1)).^2 + (coords(:,2)-coordsRef(2)).^2);
% Index = find((1-e)*R<=Rr & Rr<=(1+e)*R)
CoupX = min(coords(:,1))+ rangeX*0.5;
CoupY = min(coords(:,2))+ rangeY/2;

IndexX = find((1-e) * CoupY <= coords(:,2) & coords(:,2)<= (1+e) * CoupY );
IndexY = find((1-e) * CoupX <= coords(:,1) & coords(:,1)<= (1+e) * CoupX );
coupNodes = [IndexX(2) IndexX(1) IndexY(1) IndexY(2)]; % left right bottom top

[K,M,rDofs,extendElems,extendDofs,Alpha,coordsHin] = addResonatorC(K,M,KR,MR,coords,coupNodes,coordsRn(1,:),rs,r);

case 'D'
%  get coupling nodes (horizontal)
CoupY = min(coords(:,2))+ rangeY*0.5;
IndexX = find((1-e) * CoupY <= coords(:,2) & coords(:,2)<= (1+e) * CoupY );
coupNodes = [IndexX(2) IndexX(1)];   % left right

[K,M,rDofs,extendElems,extendDofs,Alpha] = addResonatorD(K,M,KR,MR,nr,coords,coupNodes,coordsHin);  
    
end

%% Independent, dependent and free nodes

[IndNodes,DepNodes,freeNodes,Cdi] = relationInd(nNodes, presNodes, boundaries);

IndDofs = getDofs(IndNodes);           % basic independent DOFs
IndDofs = [IndDofs;rDofs];              % extended DOFs
IndDof = length(IndDofs);              % number

DepDofs = getDofs(DepNodes); 
DepDof = length(DepDofs);

freeDofs =  getDofs( freeNodes );
freeDofs = [freeDofs;rDofs];
freeDof = length(freeDofs);

Cdi = [Cdi zeros(DepDof,length(rDofs))];           % extended Cdi


%% partition K and M as [ii,id;di,dd] 
Kii = K(IndDofs,IndDofs);
Kid = K(IndDofs,DepDofs);
Kdi = K(DepDofs,IndDofs);
Kdd = K(DepDofs,DepDofs);

Mii = M(IndDofs,IndDofs);
Mid = M(IndDofs,DepDofs);
Mdi = M(DepDofs,IndDofs);
Mdd = M(DepDofs,DepDofs);

% remove dependent nodes (KEff*ui + MEff*ai = fiEff)
KEff = Kii +Kid *Cdi + Cdi'*Kdi +Cdi'*(Kdd*Cdi);
MEff = Mii +Mid *Cdi + Cdi'*Mdi +Cdi'*(Mdd*Cdi);


%% Partition KEff and MEff as[pp,pf;fp,ff]

% partition KEff
Kpp = KEff(1:presDof,1:presDof);
Kpf = KEff(1:presDof,presDof+1:end);
Kfp = KEff(presDof+1:end,1:presDof);
Kff = KEff(presDof+1:end,presDof+1:end);

% partition MEff
Mpp = MEff(1:presDof,1:presDof);
Mpf = MEff(1:presDof,presDof+1:end);
Mfp = MEff(presDof+1:end,1:presDof);
Mff = MEff(presDof+1:end,presDof+1:end);


%% Quasistatic response (Kqs*up + Mqs*ap = fiextEffp)

% get static condensate
S = -inv(Kff) * Kfp;

% get Kqs (i.e.KM) and Mqs (i.e.MM)
Kqs = Kpp + Kpf*S;
Mqs = S'*Mff*S + 2*Mpf*S + Mpp;


%% Internal dynamics ([udynf; up]=[fai;0]*eta)

% eignenfrequency (high to low) and eigenmodes (vectors)
[Fai,Omega2] = eigs(Kff,Mff,nModes+eModes,'sm');
dispM = sqrt(real(Fai'*Mff*Fai));                     % modal displacement

% normalize
for i = 1:length(dispM)
    Fai(:,i) = Fai(:,i)/dispM(i,i); 
end

% sort (low to high frequency)
[Omega,index] = sort(real(sqrt(diag(Omega2)))/2/pi); % Hz
Fai = Fai(:,index);

% filter too low frequency
[~,index] = find(Omega'>=freqTh);
Omega = Omega(index,:);
Fai = Fai(:,index);


%% Bloch analysis

% Remove the corner nodes from boundaryNodes (left,right,bottom,top)
boundaryNodes = boundaries;
boundaryNodes{1}(1) = [];
boundaryNodes{2}(1) = [];

% Bloch analysis
[Normk, Freq, U] = blochAnalysis(K,M,boundaryNodes,W,nNodes,rDofs,extendDofs,key_cons,DirWave,Nset,eModes);

% plot modes on the undeformed structure
[XR,YR,Ur]=funPlotModes(key_R,Ramp,coordsRn,coordsHin,nr,r,rangeX,rangeY,U,elems,coords,extendElems,config,Size);

%% Computational homogenization

[CMO,DMO,rhoM,H,jh,NormkH, FreqH,DynParas,CS,DS,rhoS] = compHom(W,V,coords,coordsRef,presNodes,Kqs,Mqs,Mff,Mpf,S,Nset,Omega,Fai,DirWave);

% dynamic parameters
Col = linspace(0,0.8,4);                             % color
Col =[Col;Col;Col]';
figure
subplot(1,2,1)
hold on
for n = 1:4
    plot(DynParas.rho(n,:)*1e-3,DynParas.omega/2/pi,'ko','linewidth',1,'MarkerEdgeColor','none','MarkerFaceColor',Col(n,:),'MarkerSize',3)
    
end
hold off
xlabel('\rho [10^3kg/m^3]','FontName','Times New Roman','FontWeight','Bold','FontSize',14);
ylabel('\omega [Hz]','FontName','Times New Roman','FontWeight','Bold','FontSize',14);
xlim([-1 1]*rhoR(1)*1e-3)
ylim([0 Freqlim])
set(gca,'FontName','Times New Roman','FontSize',14)
legend('e_1e_1','e_2e_2','e_1e_2','e_2e_1')
subplot(1,2,2)
hold on
for n = 1:4
    plot(DynParas.C(n,:)*1e-9,DynParas.omega/2/pi,'ko','linewidth',1,'MarkerEdgeColor','none','MarkerFaceColor',Col(n,:),'MarkerSize',3)
    
end
hold off
xlabel('C [GPa]','FontName','Times New Roman','FontWeight','Bold','FontSize',14);
xlim([-5 5]*E*1e-9)
ylim([0 Freqlim])
set(gca,'FontName','Times New Roman','FontSize',14)

set(gcf,'Units','centimeters','Position',[5 5 16 14])

%% Dispersion comparison

figure
hold on

% dispersion curves by Bloch analysis
sModes = 6;      % compare the first 6 modes
for iMode = 1:sModes
    plot(Normk,Freq(iMode,:),'b-','linewidth',3)
%     patch([Normk nan],[Freq(iMode,:) nan],[Freq(iMode,:) nan],'linewidth',3,'edgecolor','interp')
end
% colormap('cool')

% dispersion curves by homogenization model
nn = 2;
for n = 1:nn:4
plot(NormkH(n,:),FreqH(n,:),'ro','linewidth',1,'MarkerEdgeColor','none','MarkerFaceColor','r','MarkerSize',3)

end

for n = 2:nn:4
plot(NormkH(n,:),FreqH(n,:),'ro','linewidth',1,'MarkerEdgeColor','none','MarkerFaceColor','r','MarkerSize',3)

end

hold off
xlabel('\it{kL}/\pi','FontName','Times New Roman','FontWeight','Bold','FontSize',14);
ylabel('\omega [Hz]','FontName','Times New Roman','FontWeight','Bold','FontSize',14);
set(gca,'FontName','Times New Roman','FontSize',14)
% 
title('Dispersion curves','FontName','Times New Roman','FontWeight','Bold','FontSize',14)
ylim([0 Freqlim])
grid

switch RANGE
case '0 to 1'
xlim([0 1])
set(gcf,'Units','centimeters','Position',[5 5 8.5 14])

case '-1 to 1'
axis('square')
xlim([-1 1])
set(gcf,'Units','centimeters','Position',[5 5 16 14])
end





